// src/Session.hh
// (C) 2005 - 2017 Philip Endecott
// This file is part of Anyterm; see http://anyterm.org/

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifndef Session_hh
#define Session_hh

//#define __STDC_LIMIT_MACROS
//#include <stdint.h>

#include "SessionId.hh"

#include <vector>
#include <string>
#include <memory>
#include <functional>
#include <atomic>

#include <boost/noncopyable.hpp>

#include "Screen.hh"
#include "Terminal.hh"
#include "SubProcess.hh"
#include "Iconver.hh"
#include "safe_mutex.hh"


class Session: boost::noncopyable {
public:

  const SessionId id;
  const int scrollback;
  const int time_out;
  const std::string charset;
  const bool diff;

private:
  pbe::Iconver<pbe::permissive,utf8_char,char> utf8_to_charset;
  pbe::Iconver<pbe::permissive,char,ucs4_char> charset_to_ucs4;
  pbe::Iconver<pbe::valid,ucs4_char,utf8_char> ucs4_to_utf8;

public:
  typedef safe::mutex screen_mut_t;
  typedef safe::unique_lock<screen_mut_t> screen_lock_t;
  screen_mut_t screen_mut;
  typedef safe::condition_variable dirty_condition_t;
  dirty_condition_t dirty_condition;

  Screen screen         GUARDED_BY(screen_mut);
  bool dirty            GUARDED_BY(screen_mut);
  std::string error_msg GUARDED_BY(screen_mut);
  bool error            GUARDED_BY(screen_mut);
  Terminal term         GUARDED_BY(screen_mut);

  ucs4_string old_html_screen;
  std::atomic<time_t> last_access;

  SubProcess subprocess;

  Session(int r, int c, int sb,
          std::string host, std::string user, std::string param,
          int t,
          std::string command,
          std::string charset_,
          bool diff_);
  ~Session();

  void touch(void);
  void report_any_backend_error(void);
  void send(std::string k);
  std::string rcv(void);
  void resize(int rows, int cols);

  bool timed_out(void);

private:
  void process_output(std::string s);
  void process_error(std::string s);
};


#endif
